# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .aggregation import Aggregation, Aggregation_Count, Aggregation_Last, Aggregation_Max, Aggregation_Sum
    from .amount import Amount
    from .billing_state_response import BillingStateResponse
    from .count_aggregation_pricing_metric_interface import CountAggregationPricingMetricInterface
    from .count_aggregation_pricing_metric_resource import CountAggregationPricingMetricResource
    from .create_customer_portal_session_response import CreateCustomerPortalSessionResponse
    from .create_fixed_rate_request import CreateFixedRateRequest
    from .create_simple_usage_based_rate_request import CreateSimpleUsageBasedRateRequest
    from .create_subject_response import CreateSubjectResponse
    from .create_subscription_checkout_session_response import CreateSubscriptionCheckoutSessionResponse
    from .custom_aggregation_pricing_metric_interface import CustomAggregationPricingMetricInterface
    from .fixed_rate_interface import FixedRateInterface
    from .flat_price import FlatPrice
    from .http_validation_error import HttpValidationError
    from .invoice_line_item_resource import InvoiceLineItemResource
    from .invoice_resource import InvoiceResource
    from .invoice_status import InvoiceStatus
    from .last_aggregation_pricing_metric_interface import LastAggregationPricingMetricInterface
    from .last_aggregation_pricing_metric_resource import LastAggregationPricingMetricResource
    from .list_invoices_response import ListInvoicesResponse
    from .list_pricing_metrics_response import ListPricingMetricsResponse
    from .list_rate_cards_response import ListRateCardsResponse
    from .list_subjects_response import ListSubjectsResponse
    from .list_subscriptions_response import ListSubscriptionsResponse
    from .max_aggregation_pricing_metric_interface import MaxAggregationPricingMetricInterface
    from .max_aggregation_pricing_metric_resource import MaxAggregationPricingMetricResource
    from .package_price import PackagePrice
    from .package_price_input_rounding_behavior import PackagePriceInputRoundingBehavior
    from .package_price_output_rounding_behavior import PackagePriceOutputRoundingBehavior
    from .period import Period
    from .period_resource import PeriodResource
    from .price import Price, Price_Flat, Price_Package
    from .pricing_metric_resource import PricingMetricResource
    from .pricing_metric_summary_resource import PricingMetricSummaryResource
    from .rate_card_resource import RateCardResource
    from .rate_card_resource_billing_interval import RateCardResourceBillingInterval
    from .rate_card_resource_usage_based_rates_item import (
        RateCardResourceUsageBasedRatesItem,
        RateCardResourceUsageBasedRatesItem_Simple,
    )
    from .simple_usage_based_rate_interface import SimpleUsageBasedRateInterface
    from .subject_resource import SubjectResource
    from .subscription_resource import SubscriptionResource
    from .subscription_status import SubscriptionStatus
    from .sum_aggregation_pricing_metric_interface import SumAggregationPricingMetricInterface
    from .sum_aggregation_pricing_metric_resource import SumAggregationPricingMetricResource
    from .validation_error import ValidationError
    from .validation_error_loc_item import ValidationErrorLocItem
_dynamic_imports: typing.Dict[str, str] = {
    "Aggregation": ".aggregation",
    "Aggregation_Count": ".aggregation",
    "Aggregation_Last": ".aggregation",
    "Aggregation_Max": ".aggregation",
    "Aggregation_Sum": ".aggregation",
    "Amount": ".amount",
    "BillingStateResponse": ".billing_state_response",
    "CountAggregationPricingMetricInterface": ".count_aggregation_pricing_metric_interface",
    "CountAggregationPricingMetricResource": ".count_aggregation_pricing_metric_resource",
    "CreateCustomerPortalSessionResponse": ".create_customer_portal_session_response",
    "CreateFixedRateRequest": ".create_fixed_rate_request",
    "CreateSimpleUsageBasedRateRequest": ".create_simple_usage_based_rate_request",
    "CreateSubjectResponse": ".create_subject_response",
    "CreateSubscriptionCheckoutSessionResponse": ".create_subscription_checkout_session_response",
    "CustomAggregationPricingMetricInterface": ".custom_aggregation_pricing_metric_interface",
    "FixedRateInterface": ".fixed_rate_interface",
    "FlatPrice": ".flat_price",
    "HttpValidationError": ".http_validation_error",
    "InvoiceLineItemResource": ".invoice_line_item_resource",
    "InvoiceResource": ".invoice_resource",
    "InvoiceStatus": ".invoice_status",
    "LastAggregationPricingMetricInterface": ".last_aggregation_pricing_metric_interface",
    "LastAggregationPricingMetricResource": ".last_aggregation_pricing_metric_resource",
    "ListInvoicesResponse": ".list_invoices_response",
    "ListPricingMetricsResponse": ".list_pricing_metrics_response",
    "ListRateCardsResponse": ".list_rate_cards_response",
    "ListSubjectsResponse": ".list_subjects_response",
    "ListSubscriptionsResponse": ".list_subscriptions_response",
    "MaxAggregationPricingMetricInterface": ".max_aggregation_pricing_metric_interface",
    "MaxAggregationPricingMetricResource": ".max_aggregation_pricing_metric_resource",
    "PackagePrice": ".package_price",
    "PackagePriceInputRoundingBehavior": ".package_price_input_rounding_behavior",
    "PackagePriceOutputRoundingBehavior": ".package_price_output_rounding_behavior",
    "Period": ".period",
    "PeriodResource": ".period_resource",
    "Price": ".price",
    "Price_Flat": ".price",
    "Price_Package": ".price",
    "PricingMetricResource": ".pricing_metric_resource",
    "PricingMetricSummaryResource": ".pricing_metric_summary_resource",
    "RateCardResource": ".rate_card_resource",
    "RateCardResourceBillingInterval": ".rate_card_resource_billing_interval",
    "RateCardResourceUsageBasedRatesItem": ".rate_card_resource_usage_based_rates_item",
    "RateCardResourceUsageBasedRatesItem_Simple": ".rate_card_resource_usage_based_rates_item",
    "SimpleUsageBasedRateInterface": ".simple_usage_based_rate_interface",
    "SubjectResource": ".subject_resource",
    "SubscriptionResource": ".subscription_resource",
    "SubscriptionStatus": ".subscription_status",
    "SumAggregationPricingMetricInterface": ".sum_aggregation_pricing_metric_interface",
    "SumAggregationPricingMetricResource": ".sum_aggregation_pricing_metric_resource",
    "ValidationError": ".validation_error",
    "ValidationErrorLocItem": ".validation_error_loc_item",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Aggregation",
    "Aggregation_Count",
    "Aggregation_Last",
    "Aggregation_Max",
    "Aggregation_Sum",
    "Amount",
    "BillingStateResponse",
    "CountAggregationPricingMetricInterface",
    "CountAggregationPricingMetricResource",
    "CreateCustomerPortalSessionResponse",
    "CreateFixedRateRequest",
    "CreateSimpleUsageBasedRateRequest",
    "CreateSubjectResponse",
    "CreateSubscriptionCheckoutSessionResponse",
    "CustomAggregationPricingMetricInterface",
    "FixedRateInterface",
    "FlatPrice",
    "HttpValidationError",
    "InvoiceLineItemResource",
    "InvoiceResource",
    "InvoiceStatus",
    "LastAggregationPricingMetricInterface",
    "LastAggregationPricingMetricResource",
    "ListInvoicesResponse",
    "ListPricingMetricsResponse",
    "ListRateCardsResponse",
    "ListSubjectsResponse",
    "ListSubscriptionsResponse",
    "MaxAggregationPricingMetricInterface",
    "MaxAggregationPricingMetricResource",
    "PackagePrice",
    "PackagePriceInputRoundingBehavior",
    "PackagePriceOutputRoundingBehavior",
    "Period",
    "PeriodResource",
    "Price",
    "Price_Flat",
    "Price_Package",
    "PricingMetricResource",
    "PricingMetricSummaryResource",
    "RateCardResource",
    "RateCardResourceBillingInterval",
    "RateCardResourceUsageBasedRatesItem",
    "RateCardResourceUsageBasedRatesItem_Simple",
    "SimpleUsageBasedRateInterface",
    "SubjectResource",
    "SubscriptionResource",
    "SubscriptionStatus",
    "SumAggregationPricingMetricInterface",
    "SumAggregationPricingMetricResource",
    "ValidationError",
    "ValidationErrorLocItem",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .fixed_rate_interface import FixedRateInterface
from .rate_card_resource_billing_interval import RateCardResourceBillingInterval
from .rate_card_resource_usage_based_rates_item import RateCardResourceUsageBasedRatesItem


class RateCardResource(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the rate card.
    """

    name: str = pydantic.Field()
    """
    The name of the rate card.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the rate card.
    """

    usage_based_rates: typing.List[RateCardResourceUsageBasedRatesItem] = pydantic.Field()
    """
    The usage based rates of the rate card.
    """

    fixed_rates: typing.List[FixedRateInterface] = pydantic.Field()
    """
    The fixed rates of the rate card.
    """

    billing_interval: RateCardResourceBillingInterval
    created_at: dt.datetime = pydantic.Field()
    """
    The date and time the rate card was created.
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    The date and time the rate card was last updated.
    """

    metadata: typing.Dict[str, str]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawUsageEventsClient, RawUsageEventsClient
from .types.create_usage_event_request_data_value import CreateUsageEventRequestDataValue

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsageEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsageEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsageEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsageEventsClient
        """
        return self._raw_client

    def create_usage_event(
        self,
        *,
        idempotency_key: str,
        event_name: str,
        subject_id: str,
        data: typing.Dict[str, CreateUsageEventRequestDataValue],
        timestamp: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        idempotency_key : str
            The idempotency key for the usage event. This ensures that the same event is not processed multiple times.

        event_name : str
            The name of the event. This is used by pricing metrics to aggregate usage events.

        subject_id : str
            The ID of the subject that the usage event is for.

        data : typing.Dict[str, CreateUsageEventRequestDataValue]
            The data of the usage event. This should contain any data that is needed to aggregate the usage event.

        timestamp : typing.Optional[dt.datetime]
            The timestamp of the usage event. It is highly recommended to provide a timestamp. If not provided, the current timestamp will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import datetime

        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.usage_events.create_usage_event(
            idempotency_key="6b4ef298-4566-427d-b2ca-cd8fbe38ec60",
            event_name="compute_hours",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            timestamp=datetime.datetime.fromisoformat(
                "2025-10-31 20:29:47+00:00",
            ),
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
        )
        """
        _response = self._raw_client.create_usage_event(
            idempotency_key=idempotency_key,
            event_name=event_name,
            subject_id=subject_id,
            data=data,
            timestamp=timestamp,
            request_options=request_options,
        )
        return _response.data


class AsyncUsageEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsageEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsageEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsageEventsClient
        """
        return self._raw_client

    async def create_usage_event(
        self,
        *,
        idempotency_key: str,
        event_name: str,
        subject_id: str,
        data: typing.Dict[str, CreateUsageEventRequestDataValue],
        timestamp: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        idempotency_key : str
            The idempotency key for the usage event. This ensures that the same event is not processed multiple times.

        event_name : str
            The name of the event. This is used by pricing metrics to aggregate usage events.

        subject_id : str
            The ID of the subject that the usage event is for.

        data : typing.Dict[str, CreateUsageEventRequestDataValue]
            The data of the usage event. This should contain any data that is needed to aggregate the usage event.

        timestamp : typing.Optional[dt.datetime]
            The timestamp of the usage event. It is highly recommended to provide a timestamp. If not provided, the current timestamp will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.usage_events.create_usage_event(
                idempotency_key="6b4ef298-4566-427d-b2ca-cd8fbe38ec60",
                event_name="compute_hours",
                subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
                timestamp=datetime.datetime.fromisoformat(
                    "2025-10-31 20:29:47+00:00",
                ),
                data={
                    "compute_hours": 100,
                    "instance_type": "t2.micro",
                    "region": "us-east-1",
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_usage_event(
            idempotency_key=idempotency_key,
            event_name=event_name,
            subject_id=subject_id,
            data=data,
            timestamp=timestamp,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .amount import Amount
from .invoice_line_item_resource import InvoiceLineItemResource
from .invoice_status import InvoiceStatus


class InvoiceResource(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the invoice.
    """

    total_amount: Amount = pydantic.Field()
    """
    The total amount of the invoice.
    """

    subject_id: str = pydantic.Field()
    """
    The ID of the subject for the invoice.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The date and time the invoice was created.
    """

    status: InvoiceStatus = pydantic.Field()
    """
    The status of the invoice.
    """

    hosted_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL of the hosted invoice.
    """

    line_items: typing.List[InvoiceLineItemResource] = pydantic.Field()
    """
    The line items of the invoice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Aggregation_Count(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["count"] = "count"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Aggregation_Last(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["last"] = "last"
    value_field: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Aggregation_Max(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["max"] = "max"
    value_field: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Aggregation_Sum(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["sum"] = "sum"
    value_field: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


Aggregation = typing.Union[Aggregation_Count, Aggregation_Last, Aggregation_Max, Aggregation_Sum]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.list_invoices_response import ListInvoicesResponse
from .raw_client import AsyncRawInvoicesClient, RawInvoicesClient


class InvoicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawInvoicesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawInvoicesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawInvoicesClient
        """
        return self._raw_client

    def list_invoices(
        self,
        *,
        subject_id: str,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListInvoicesResponse:
        """
        Parameters
        ----------
        subject_id : str
            The ID of the subject to list invoices for.

        limit : typing.Optional[int]

        offset : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListInvoicesResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.invoices.list_invoices(
            subject_id="subject_id",
            limit=1,
            offset=1,
        )
        """
        _response = self._raw_client.list_invoices(
            subject_id=subject_id, limit=limit, offset=offset, request_options=request_options
        )
        return _response.data


class AsyncInvoicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawInvoicesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawInvoicesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawInvoicesClient
        """
        return self._raw_client

    async def list_invoices(
        self,
        *,
        subject_id: str,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListInvoicesResponse:
        """
        Parameters
        ----------
        subject_id : str
            The ID of the subject to list invoices for.

        limit : typing.Optional[int]

        offset : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListInvoicesResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoices.list_invoices(
                subject_id="subject_id",
                limit=1,
                offset=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_invoices(
            subject_id=subject_id, limit=limit, offset=offset, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import httpx
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import LarkEnvironment

if typing.TYPE_CHECKING:
    from .checkout.client import AsyncCheckoutClient, CheckoutClient
    from .customer_access.client import AsyncCustomerAccessClient, CustomerAccessClient
    from .customer_portal.client import AsyncCustomerPortalClient, CustomerPortalClient
    from .invoices.client import AsyncInvoicesClient, InvoicesClient
    from .pricing_metrics.client import AsyncPricingMetricsClient, PricingMetricsClient
    from .rate_cards.client import AsyncRateCardsClient, RateCardsClient
    from .subjects.client import AsyncSubjectsClient, SubjectsClient
    from .subscriptions.client import AsyncSubscriptionsClient, SubscriptionsClient
    from .usage_events.client import AsyncUsageEventsClient, UsageEventsClient


class Lark:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : LarkEnvironment
        The environment to use for requests from the client. from .environment import LarkEnvironment



        Defaults to LarkEnvironment.DEFAULT



    api_key : str
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from lark import Lark

    client = Lark(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: LarkEnvironment = LarkEnvironment.DEFAULT,
        api_key: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._checkout: typing.Optional[CheckoutClient] = None
        self._customer_portal: typing.Optional[CustomerPortalClient] = None
        self._rate_cards: typing.Optional[RateCardsClient] = None
        self._usage_events: typing.Optional[UsageEventsClient] = None
        self._subscriptions: typing.Optional[SubscriptionsClient] = None
        self._subjects: typing.Optional[SubjectsClient] = None
        self._pricing_metrics: typing.Optional[PricingMetricsClient] = None
        self._customer_access: typing.Optional[CustomerAccessClient] = None
        self._invoices: typing.Optional[InvoicesClient] = None

    @property
    def checkout(self):
        if self._checkout is None:
            from .checkout.client import CheckoutClient  # noqa: E402

            self._checkout = CheckoutClient(client_wrapper=self._client_wrapper)
        return self._checkout

    @property
    def customer_portal(self):
        if self._customer_portal is None:
            from .customer_portal.client import CustomerPortalClient  # noqa: E402

            self._customer_portal = CustomerPortalClient(client_wrapper=self._client_wrapper)
        return self._customer_portal

    @property
    def rate_cards(self):
        if self._rate_cards is None:
            from .rate_cards.client import RateCardsClient  # noqa: E402

            self._rate_cards = RateCardsClient(client_wrapper=self._client_wrapper)
        return self._rate_cards

    @property
    def usage_events(self):
        if self._usage_events is None:
            from .usage_events.client import UsageEventsClient  # noqa: E402

            self._usage_events = UsageEventsClient(client_wrapper=self._client_wrapper)
        return self._usage_events

    @property
    def subscriptions(self):
        if self._subscriptions is None:
            from .subscriptions.client import SubscriptionsClient  # noqa: E402

            self._subscriptions = SubscriptionsClient(client_wrapper=self._client_wrapper)
        return self._subscriptions

    @property
    def subjects(self):
        if self._subjects is None:
            from .subjects.client import SubjectsClient  # noqa: E402

            self._subjects = SubjectsClient(client_wrapper=self._client_wrapper)
        return self._subjects

    @property
    def pricing_metrics(self):
        if self._pricing_metrics is None:
            from .pricing_metrics.client import PricingMetricsClient  # noqa: E402

            self._pricing_metrics = PricingMetricsClient(client_wrapper=self._client_wrapper)
        return self._pricing_metrics

    @property
    def customer_access(self):
        if self._customer_access is None:
            from .customer_access.client import CustomerAccessClient  # noqa: E402

            self._customer_access = CustomerAccessClient(client_wrapper=self._client_wrapper)
        return self._customer_access

    @property
    def invoices(self):
        if self._invoices is None:
            from .invoices.client import InvoicesClient  # noqa: E402

            self._invoices = InvoicesClient(client_wrapper=self._client_wrapper)
        return self._invoices


class AsyncLark:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : LarkEnvironment
        The environment to use for requests from the client. from .environment import LarkEnvironment



        Defaults to LarkEnvironment.DEFAULT



    api_key : str
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from lark import AsyncLark

    client = AsyncLark(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: LarkEnvironment = LarkEnvironment.DEFAULT,
        api_key: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._checkout: typing.Optional[AsyncCheckoutClient] = None
        self._customer_portal: typing.Optional[AsyncCustomerPortalClient] = None
        self._rate_cards: typing.Optional[AsyncRateCardsClient] = None
        self._usage_events: typing.Optional[AsyncUsageEventsClient] = None
        self._subscriptions: typing.Optional[AsyncSubscriptionsClient] = None
        self._subjects: typing.Optional[AsyncSubjectsClient] = None
        self._pricing_metrics: typing.Optional[AsyncPricingMetricsClient] = None
        self._customer_access: typing.Optional[AsyncCustomerAccessClient] = None
        self._invoices: typing.Optional[AsyncInvoicesClient] = None

    @property
    def checkout(self):
        if self._checkout is None:
            from .checkout.client import AsyncCheckoutClient  # noqa: E402

            self._checkout = AsyncCheckoutClient(client_wrapper=self._client_wrapper)
        return self._checkout

    @property
    def customer_portal(self):
        if self._customer_portal is None:
            from .customer_portal.client import AsyncCustomerPortalClient  # noqa: E402

            self._customer_portal = AsyncCustomerPortalClient(client_wrapper=self._client_wrapper)
        return self._customer_portal

    @property
    def rate_cards(self):
        if self._rate_cards is None:
            from .rate_cards.client import AsyncRateCardsClient  # noqa: E402

            self._rate_cards = AsyncRateCardsClient(client_wrapper=self._client_wrapper)
        return self._rate_cards

    @property
    def usage_events(self):
        if self._usage_events is None:
            from .usage_events.client import AsyncUsageEventsClient  # noqa: E402

            self._usage_events = AsyncUsageEventsClient(client_wrapper=self._client_wrapper)
        return self._usage_events

    @property
    def subscriptions(self):
        if self._subscriptions is None:
            from .subscriptions.client import AsyncSubscriptionsClient  # noqa: E402

            self._subscriptions = AsyncSubscriptionsClient(client_wrapper=self._client_wrapper)
        return self._subscriptions

    @property
    def subjects(self):
        if self._subjects is None:
            from .subjects.client import AsyncSubjectsClient  # noqa: E402

            self._subjects = AsyncSubjectsClient(client_wrapper=self._client_wrapper)
        return self._subjects

    @property
    def pricing_metrics(self):
        if self._pricing_metrics is None:
            from .pricing_metrics.client import AsyncPricingMetricsClient  # noqa: E402

            self._pricing_metrics = AsyncPricingMetricsClient(client_wrapper=self._client_wrapper)
        return self._pricing_metrics

    @property
    def customer_access(self):
        if self._customer_access is None:
            from .customer_access.client import AsyncCustomerAccessClient  # noqa: E402

            self._customer_access = AsyncCustomerAccessClient(client_wrapper=self._client_wrapper)
        return self._customer_access

    @property
    def invoices(self):
        if self._invoices is None:
            from .invoices.client import AsyncInvoicesClient  # noqa: E402

            self._invoices = AsyncInvoicesClient(client_wrapper=self._client_wrapper)
        return self._invoices


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: LarkEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_subject_response import CreateSubjectResponse
from ..types.list_subjects_response import ListSubjectsResponse
from ..types.subject_resource import SubjectResource
from .raw_client import AsyncRawSubjectsClient, RawSubjectsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SubjectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSubjectsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSubjectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSubjectsClient
        """
        return self._raw_client

    def list_subjects(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSubjectsResponse:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        offset : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSubjectsResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subjects.list_subjects(
            limit=1,
            offset=1,
        )
        """
        _response = self._raw_client.list_subjects(limit=limit, offset=offset, request_options=request_options)
        return _response.data

    def create_subject(
        self,
        *,
        external_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSubjectResponse:
        """
        Parameters
        ----------
        external_id : typing.Optional[str]
            The ID of the subject in your system. If provided, you may use pass it to the API in place of the subject ID. Must be unique.

        name : typing.Optional[str]
            The name of the subject. Used for display in the dashboard.

        email : typing.Optional[str]
            The email of the subject. Must be a valid email address.

        metadata : typing.Optional[typing.Dict[str, str]]
            Additional metadata about the subject. You may use this to store any custom data about the subject.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSubjectResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subjects.create_subject(
            external_id="user_1234567890",
            name="John Doe",
            email="john.doe@example.com",
            metadata={"key": "value"},
        )
        """
        _response = self._raw_client.create_subject(
            external_id=external_id, name=name, email=email, metadata=metadata, request_options=request_options
        )
        return _response.data

    def get_subject(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubjectResource:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubjectResource
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subjects.get_subject(
            subject_id="subject_id",
        )
        """
        _response = self._raw_client.get_subject(subject_id, request_options=request_options)
        return _response.data

    def update_subject(
        self,
        subject_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SubjectResource:
        """
        Parameters
        ----------
        subject_id : str

        name : typing.Optional[str]
            The name of the subject. Used for display in the dashboard.

        email : typing.Optional[str]
            The email of the subject. Must be a valid email address.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional metadata about the subject. You may use this to store any custom data about the subject.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubjectResource
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subjects.update_subject(
            subject_id="subject_id",
        )
        """
        _response = self._raw_client.update_subject(
            subject_id, name=name, email=email, metadata=metadata, request_options=request_options
        )
        return _response.data

    def delete_subject(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subjects.delete_subject(
            subject_id="subject_id",
        )
        """
        _response = self._raw_client.delete_subject(subject_id, request_options=request_options)
        return _response.data


class AsyncSubjectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSubjectsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSubjectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSubjectsClient
        """
        return self._raw_client

    async def list_subjects(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSubjectsResponse:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        offset : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSubjectsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subjects.list_subjects(
                limit=1,
                offset=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_subjects(limit=limit, offset=offset, request_options=request_options)
        return _response.data

    async def create_subject(
        self,
        *,
        external_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSubjectResponse:
        """
        Parameters
        ----------
        external_id : typing.Optional[str]
            The ID of the subject in your system. If provided, you may use pass it to the API in place of the subject ID. Must be unique.

        name : typing.Optional[str]
            The name of the subject. Used for display in the dashboard.

        email : typing.Optional[str]
            The email of the subject. Must be a valid email address.

        metadata : typing.Optional[typing.Dict[str, str]]
            Additional metadata about the subject. You may use this to store any custom data about the subject.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSubjectResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subjects.create_subject(
                external_id="user_1234567890",
                name="John Doe",
                email="john.doe@example.com",
                metadata={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_subject(
            external_id=external_id, name=name, email=email, metadata=metadata, request_options=request_options
        )
        return _response.data

    async def get_subject(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubjectResource:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubjectResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subjects.get_subject(
                subject_id="subject_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_subject(subject_id, request_options=request_options)
        return _response.data

    async def update_subject(
        self,
        subject_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SubjectResource:
        """
        Parameters
        ----------
        subject_id : str

        name : typing.Optional[str]
            The name of the subject. Used for display in the dashboard.

        email : typing.Optional[str]
            The email of the subject. Must be a valid email address.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional metadata about the subject. You may use this to store any custom data about the subject.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubjectResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subjects.update_subject(
                subject_id="subject_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_subject(
            subject_id, name=name, email=email, metadata=metadata, request_options=request_options
        )
        return _response.data

    async def delete_subject(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subjects.delete_subject(
                subject_id="subject_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_subject(subject_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.billing_state_response import BillingStateResponse
from .raw_client import AsyncRawCustomerAccessClient, RawCustomerAccessClient


class CustomerAccessClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomerAccessClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomerAccessClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomerAccessClient
        """
        return self._raw_client

    def get_billing_state(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BillingStateResponse:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillingStateResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.customer_access.get_billing_state(
            subject_id="subject_id",
        )
        """
        _response = self._raw_client.get_billing_state(subject_id, request_options=request_options)
        return _response.data


class AsyncCustomerAccessClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomerAccessClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomerAccessClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomerAccessClient
        """
        return self._raw_client

    async def get_billing_state(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BillingStateResponse:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillingStateResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.customer_access.get_billing_state(
                subject_id="subject_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_billing_state(subject_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_customer_portal_session_response import CreateCustomerPortalSessionResponse
from .raw_client import AsyncRawCustomerPortalClient, RawCustomerPortalClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomerPortalClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomerPortalClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomerPortalClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomerPortalClient
        """
        return self._raw_client

    def create_customer_portal_session(
        self, *, subject_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateCustomerPortalSessionResponse:
        """
        Parameters
        ----------
        subject_id : str
            The ID of the subject to create the customer portal session for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerPortalSessionResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.customer_portal.create_customer_portal_session(
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )
        """
        _response = self._raw_client.create_customer_portal_session(
            subject_id=subject_id, request_options=request_options
        )
        return _response.data


class AsyncCustomerPortalClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomerPortalClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomerPortalClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomerPortalClient
        """
        return self._raw_client

    async def create_customer_portal_session(
        self, *, subject_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateCustomerPortalSessionResponse:
        """
        Parameters
        ----------
        subject_id : str
            The ID of the subject to create the customer portal session for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerPortalSessionResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.customer_portal.create_customer_portal_session(
                subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_customer_portal_session(
            subject_id=subject_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CreateSubjectResponse(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the subject.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the subject in your system. You may pass it to the API in place of the subject ID.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the subject. Used for display in the dashboard.
    """

    email: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email of the subject.
    """

    metadata: typing.Dict[str, str] = pydantic.Field()
    """
    Additional metadata about the subject. You may use this to store any custom data about the subject.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The date and time the subject was created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Aggregation,
        Aggregation_Count,
        Aggregation_Last,
        Aggregation_Max,
        Aggregation_Sum,
        Amount,
        BillingStateResponse,
        CountAggregationPricingMetricInterface,
        CountAggregationPricingMetricResource,
        CreateCustomerPortalSessionResponse,
        CreateFixedRateRequest,
        CreateSimpleUsageBasedRateRequest,
        CreateSubjectResponse,
        CreateSubscriptionCheckoutSessionResponse,
        CustomAggregationPricingMetricInterface,
        FixedRateInterface,
        FlatPrice,
        HttpValidationError,
        InvoiceLineItemResource,
        InvoiceResource,
        InvoiceStatus,
        LastAggregationPricingMetricInterface,
        LastAggregationPricingMetricResource,
        ListInvoicesResponse,
        ListPricingMetricsResponse,
        ListRateCardsResponse,
        ListSubjectsResponse,
        ListSubscriptionsResponse,
        MaxAggregationPricingMetricInterface,
        MaxAggregationPricingMetricResource,
        PackagePrice,
        PackagePriceInputRoundingBehavior,
        PackagePriceOutputRoundingBehavior,
        Period,
        PeriodResource,
        Price,
        Price_Flat,
        Price_Package,
        PricingMetricResource,
        PricingMetricSummaryResource,
        RateCardResource,
        RateCardResourceBillingInterval,
        RateCardResourceUsageBasedRatesItem,
        RateCardResourceUsageBasedRatesItem_Simple,
        SimpleUsageBasedRateInterface,
        SubjectResource,
        SubscriptionResource,
        SubscriptionStatus,
        SumAggregationPricingMetricInterface,
        SumAggregationPricingMetricResource,
        ValidationError,
        ValidationErrorLocItem,
    )
    from .errors import UnprocessableEntityError
    from . import (
        checkout,
        customer_access,
        customer_portal,
        invoices,
        pricing_metrics,
        rate_cards,
        subjects,
        subscriptions,
        usage_events,
    )
    from .client import AsyncLark, Lark
    from .environment import LarkEnvironment
    from .pricing_metrics import (
        PricingMetricAggregation,
        PricingMetricAggregation_Count,
        PricingMetricAggregation_Custom,
        PricingMetricAggregation_Last,
        PricingMetricAggregation_Max,
        PricingMetricAggregation_Sum,
    )
    from .rate_cards import (
        CreateRateCardRequestBillingInterval,
        CreateRateCardRequestUsageBasedRatesItem,
        CreateRateCardRequestUsageBasedRatesItem_Simple,
    )
    from .usage_events import CreateUsageEventRequestDataValue
    from .version import __version__
_dynamic_imports: typing.Dict[str, str] = {
    "Aggregation": ".types",
    "Aggregation_Count": ".types",
    "Aggregation_Last": ".types",
    "Aggregation_Max": ".types",
    "Aggregation_Sum": ".types",
    "Amount": ".types",
    "AsyncLark": ".client",
    "BillingStateResponse": ".types",
    "CountAggregationPricingMetricInterface": ".types",
    "CountAggregationPricingMetricResource": ".types",
    "CreateCustomerPortalSessionResponse": ".types",
    "CreateFixedRateRequest": ".types",
    "CreateRateCardRequestBillingInterval": ".rate_cards",
    "CreateRateCardRequestUsageBasedRatesItem": ".rate_cards",
    "CreateRateCardRequestUsageBasedRatesItem_Simple": ".rate_cards",
    "CreateSimpleUsageBasedRateRequest": ".types",
    "CreateSubjectResponse": ".types",
    "CreateSubscriptionCheckoutSessionResponse": ".types",
    "CreateUsageEventRequestDataValue": ".usage_events",
    "CustomAggregationPricingMetricInterface": ".types",
    "FixedRateInterface": ".types",
    "FlatPrice": ".types",
    "HttpValidationError": ".types",
    "InvoiceLineItemResource": ".types",
    "InvoiceResource": ".types",
    "InvoiceStatus": ".types",
    "Lark": ".client",
    "LarkEnvironment": ".environment",
    "LastAggregationPricingMetricInterface": ".types",
    "LastAggregationPricingMetricResource": ".types",
    "ListInvoicesResponse": ".types",
    "ListPricingMetricsResponse": ".types",
    "ListRateCardsResponse": ".types",
    "ListSubjectsResponse": ".types",
    "ListSubscriptionsResponse": ".types",
    "MaxAggregationPricingMetricInterface": ".types",
    "MaxAggregationPricingMetricResource": ".types",
    "PackagePrice": ".types",
    "PackagePriceInputRoundingBehavior": ".types",
    "PackagePriceOutputRoundingBehavior": ".types",
    "Period": ".types",
    "PeriodResource": ".types",
    "Price": ".types",
    "Price_Flat": ".types",
    "Price_Package": ".types",
    "PricingMetricAggregation": ".pricing_metrics",
    "PricingMetricAggregation_Count": ".pricing_metrics",
    "PricingMetricAggregation_Custom": ".pricing_metrics",
    "PricingMetricAggregation_Last": ".pricing_metrics",
    "PricingMetricAggregation_Max": ".pricing_metrics",
    "PricingMetricAggregation_Sum": ".pricing_metrics",
    "PricingMetricResource": ".types",
    "PricingMetricSummaryResource": ".types",
    "RateCardResource": ".types",
    "RateCardResourceBillingInterval": ".types",
    "RateCardResourceUsageBasedRatesItem": ".types",
    "RateCardResourceUsageBasedRatesItem_Simple": ".types",
    "SimpleUsageBasedRateInterface": ".types",
    "SubjectResource": ".types",
    "SubscriptionResource": ".types",
    "SubscriptionStatus": ".types",
    "SumAggregationPricingMetricInterface": ".types",
    "SumAggregationPricingMetricResource": ".types",
    "UnprocessableEntityError": ".errors",
    "ValidationError": ".types",
    "ValidationErrorLocItem": ".types",
    "__version__": ".version",
    "checkout": ".checkout",
    "customer_access": ".customer_access",
    "customer_portal": ".customer_portal",
    "invoices": ".invoices",
    "pricing_metrics": ".pricing_metrics",
    "rate_cards": ".rate_cards",
    "subjects": ".subjects",
    "subscriptions": ".subscriptions",
    "usage_events": ".usage_events",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Aggregation",
    "Aggregation_Count",
    "Aggregation_Last",
    "Aggregation_Max",
    "Aggregation_Sum",
    "Amount",
    "AsyncLark",
    "BillingStateResponse",
    "CountAggregationPricingMetricInterface",
    "CountAggregationPricingMetricResource",
    "CreateCustomerPortalSessionResponse",
    "CreateFixedRateRequest",
    "CreateRateCardRequestBillingInterval",
    "CreateRateCardRequestUsageBasedRatesItem",
    "CreateRateCardRequestUsageBasedRatesItem_Simple",
    "CreateSimpleUsageBasedRateRequest",
    "CreateSubjectResponse",
    "CreateSubscriptionCheckoutSessionResponse",
    "CreateUsageEventRequestDataValue",
    "CustomAggregationPricingMetricInterface",
    "FixedRateInterface",
    "FlatPrice",
    "HttpValidationError",
    "InvoiceLineItemResource",
    "InvoiceResource",
    "InvoiceStatus",
    "Lark",
    "LarkEnvironment",
    "LastAggregationPricingMetricInterface",
    "LastAggregationPricingMetricResource",
    "ListInvoicesResponse",
    "ListPricingMetricsResponse",
    "ListRateCardsResponse",
    "ListSubjectsResponse",
    "ListSubscriptionsResponse",
    "MaxAggregationPricingMetricInterface",
    "MaxAggregationPricingMetricResource",
    "PackagePrice",
    "PackagePriceInputRoundingBehavior",
    "PackagePriceOutputRoundingBehavior",
    "Period",
    "PeriodResource",
    "Price",
    "Price_Flat",
    "Price_Package",
    "PricingMetricAggregation",
    "PricingMetricAggregation_Count",
    "PricingMetricAggregation_Custom",
    "PricingMetricAggregation_Last",
    "PricingMetricAggregation_Max",
    "PricingMetricAggregation_Sum",
    "PricingMetricResource",
    "PricingMetricSummaryResource",
    "RateCardResource",
    "RateCardResourceBillingInterval",
    "RateCardResourceUsageBasedRatesItem",
    "RateCardResourceUsageBasedRatesItem_Simple",
    "SimpleUsageBasedRateInterface",
    "SubjectResource",
    "SubscriptionResource",
    "SubscriptionStatus",
    "SumAggregationPricingMetricInterface",
    "SumAggregationPricingMetricResource",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
    "__version__",
    "checkout",
    "customer_access",
    "customer_portal",
    "invoices",
    "pricing_metrics",
    "rate_cards",
    "subjects",
    "subscriptions",
    "usage_events",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.list_pricing_metrics_response import ListPricingMetricsResponse
from ..types.period import Period
from ..types.pricing_metric_resource import PricingMetricResource
from ..types.pricing_metric_summary_resource import PricingMetricSummaryResource
from .raw_client import AsyncRawPricingMetricsClient, RawPricingMetricsClient
from .types.pricing_metric_aggregation import PricingMetricAggregation

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PricingMetricsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPricingMetricsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPricingMetricsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPricingMetricsClient
        """
        return self._raw_client

    def list_pricing_metrics(
        self, *, limit: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListPricingMetricsResponse:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListPricingMetricsResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.pricing_metrics.list_pricing_metrics(
            limit=1,
        )
        """
        _response = self._raw_client.list_pricing_metrics(limit=limit, request_options=request_options)
        return _response.data

    def create_pricing_metric(
        self,
        *,
        name: str,
        event_name: str,
        aggregation: PricingMetricAggregation,
        unit: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PricingMetricResource:
        """
        Parameters
        ----------
        name : str
            The name of the pricing metric.

        event_name : str
            The name of the event that the pricing metric is computed on.

        aggregation : PricingMetricAggregation
            The aggregation function used to compute the value of the pricing metric.

        unit : str
            Unit of measurement for the pricing metric.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PricingMetricResource
            Successful Response

        Examples
        --------
        from lark import Lark
        from lark.pricing_metrics import PricingMetricAggregation_Sum

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.pricing_metrics.create_pricing_metric(
            name="Compute Hours",
            event_name="job_completed",
            aggregation=PricingMetricAggregation_Sum(
                value_field="value_field",
            ),
            unit="hours",
        )
        """
        _response = self._raw_client.create_pricing_metric(
            name=name, event_name=event_name, aggregation=aggregation, unit=unit, request_options=request_options
        )
        return _response.data

    def get_pricing_metric(
        self, pricing_metric_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PricingMetricResource:
        """
        Parameters
        ----------
        pricing_metric_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PricingMetricResource
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.pricing_metrics.get_pricing_metric(
            pricing_metric_id="pricing_metric_id",
        )
        """
        _response = self._raw_client.get_pricing_metric(pricing_metric_id, request_options=request_options)
        return _response.data

    def create_pricing_metric_summary(
        self,
        pricing_metric_id: str,
        *,
        subject_id: str,
        period: Period,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PricingMetricSummaryResource:
        """
        Parameters
        ----------
        pricing_metric_id : str

        subject_id : str
            The ID of the subject that the summary should be computed for.

        period : Period
            The period that the summary should be computed over.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PricingMetricSummaryResource
            Successful Response

        Examples
        --------
        import datetime

        from lark import Lark, Period

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.pricing_metrics.create_pricing_metric_summary(
            pricing_metric_id="pricing_metric_id",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            period=Period(
                start=datetime.datetime.fromisoformat(
                    "2025-10-01 00:00:00+00:00",
                ),
                end=datetime.datetime.fromisoformat(
                    "2025-11-01 00:00:00+00:00",
                ),
            ),
        )
        """
        _response = self._raw_client.create_pricing_metric_summary(
            pricing_metric_id, subject_id=subject_id, period=period, request_options=request_options
        )
        return _response.data


class AsyncPricingMetricsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPricingMetricsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPricingMetricsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPricingMetricsClient
        """
        return self._raw_client

    async def list_pricing_metrics(
        self, *, limit: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListPricingMetricsResponse:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListPricingMetricsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pricing_metrics.list_pricing_metrics(
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_pricing_metrics(limit=limit, request_options=request_options)
        return _response.data

    async def create_pricing_metric(
        self,
        *,
        name: str,
        event_name: str,
        aggregation: PricingMetricAggregation,
        unit: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PricingMetricResource:
        """
        Parameters
        ----------
        name : str
            The name of the pricing metric.

        event_name : str
            The name of the event that the pricing metric is computed on.

        aggregation : PricingMetricAggregation
            The aggregation function used to compute the value of the pricing metric.

        unit : str
            Unit of measurement for the pricing metric.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PricingMetricResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark
        from lark.pricing_metrics import PricingMetricAggregation_Sum

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pricing_metrics.create_pricing_metric(
                name="Compute Hours",
                event_name="job_completed",
                aggregation=PricingMetricAggregation_Sum(
                    value_field="value_field",
                ),
                unit="hours",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_pricing_metric(
            name=name, event_name=event_name, aggregation=aggregation, unit=unit, request_options=request_options
        )
        return _response.data

    async def get_pricing_metric(
        self, pricing_metric_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PricingMetricResource:
        """
        Parameters
        ----------
        pricing_metric_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PricingMetricResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pricing_metrics.get_pricing_metric(
                pricing_metric_id="pricing_metric_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_pricing_metric(pricing_metric_id, request_options=request_options)
        return _response.data

    async def create_pricing_metric_summary(
        self,
        pricing_metric_id: str,
        *,
        subject_id: str,
        period: Period,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PricingMetricSummaryResource:
        """
        Parameters
        ----------
        pricing_metric_id : str

        subject_id : str
            The ID of the subject that the summary should be computed for.

        period : Period
            The period that the summary should be computed over.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PricingMetricSummaryResource
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from lark import AsyncLark, Period

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pricing_metrics.create_pricing_metric_summary(
                pricing_metric_id="pricing_metric_id",
                subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
                period=Period(
                    start=datetime.datetime.fromisoformat(
                        "2025-10-01 00:00:00+00:00",
                    ),
                    end=datetime.datetime.fromisoformat(
                        "2025-11-01 00:00:00+00:00",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_pricing_metric_summary(
            pricing_metric_id, subject_id=subject_id, period=period, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .period_resource import PeriodResource
from .subscription_status import SubscriptionStatus


class SubscriptionResource(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the subscription.
    """

    subject_id: str = pydantic.Field()
    """
    The ID of the subject that the subscription is for.
    """

    rate_card_id: str = pydantic.Field()
    """
    The ID of the rate card of the subscription.
    """

    effective_at: dt.datetime = pydantic.Field()
    """
    The date and time the subscription became effective.
    """

    cycles_next_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date and time the next cycle of the subscription will start.
    """

    current_period: typing.Optional[PeriodResource] = pydantic.Field(default=None)
    """
    The current period of the subscription if it is active.
    """

    metadata: typing.Dict[str, str]
    status: SubscriptionStatus = pydantic.Field()
    """
    The status of the subscription.
    """

    cancels_at_end_of_cycle: bool = pydantic.Field()
    """
    Whether the subscription will be cancelled at the end of the current cycle.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .period import Period


class PricingMetricSummaryResource(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the pricing metric summary.
    """

    pricing_metric_id: str = pydantic.Field()
    """
    The ID of the pricing metric that the summary is for.
    """

    subject_id: str = pydantic.Field()
    """
    The ID of the subject that the summary is for.
    """

    period: Period = pydantic.Field()
    """
    The period that the summary is computed over.
    """

    value: typing.Optional[str] = pydantic.Field(default=None)
    """
    The computed value of the pricing metric for the period. If the pricing metric does not have any usage events for the period, this will be `null`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class BillingStateResponse(UniversalBaseModel):
    has_active_subscription: bool = pydantic.Field()
    """
    Whether the subject has an active subscription.
    """

    has_overage_for_usage: bool = pydantic.Field()
    """
    Whether the subject has exceeded the included usage (if any) on a usage-based rate they are subscribed to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .price import Price


class CreateSimpleUsageBasedRateRequest(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    The name of the rate displayed to the customer.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the rate displayed to the customer.
    """

    price: Price
    included_units: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of units included in the rate before the price is applied.
    """

    pricing_metric_id: str = pydantic.Field()
    """
    The ID of the pricing metric to use for this rate.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.create_subscription_checkout_session_response import CreateSubscriptionCheckoutSessionResponse
from ..types.http_validation_error import HttpValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCheckoutClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_subscription_checkout_session(
        self,
        *,
        subject_id: str,
        rate_card_id: str,
        success_callback_url: str,
        cancel_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateSubscriptionCheckoutSessionResponse]:
        """
        Parameters
        ----------
        subject_id : str
            The ID of the subject to create the checkout for.

        rate_card_id : str
            The ID of the rate card to subscribe to.

        success_callback_url : str
            The URL to redirect to after the checkout is successful.

        cancel_url : typing.Optional[str]
            The URL to redirect to after the checkout is cancelled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateSubscriptionCheckoutSessionResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "checkout",
            method="POST",
            json={
                "subject_id": subject_id,
                "rate_card_id": rate_card_id,
                "success_callback_url": success_callback_url,
                "cancel_url": cancel_url,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSubscriptionCheckoutSessionResponse,
                    parse_obj_as(
                        type_=CreateSubscriptionCheckoutSessionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCheckoutClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_subscription_checkout_session(
        self,
        *,
        subject_id: str,
        rate_card_id: str,
        success_callback_url: str,
        cancel_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateSubscriptionCheckoutSessionResponse]:
        """
        Parameters
        ----------
        subject_id : str
            The ID of the subject to create the checkout for.

        rate_card_id : str
            The ID of the rate card to subscribe to.

        success_callback_url : str
            The URL to redirect to after the checkout is successful.

        cancel_url : typing.Optional[str]
            The URL to redirect to after the checkout is cancelled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateSubscriptionCheckoutSessionResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "checkout",
            method="POST",
            json={
                "subject_id": subject_id,
                "rate_card_id": rate_card_id,
                "success_callback_url": success_callback_url,
                "cancel_url": cancel_url,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSubscriptionCheckoutSessionResponse,
                    parse_obj_as(
                        type_=CreateSubscriptionCheckoutSessionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from .types.create_usage_event_request_data_value import CreateUsageEventRequestDataValue

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawUsageEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_usage_event(
        self,
        *,
        idempotency_key: str,
        event_name: str,
        subject_id: str,
        data: typing.Dict[str, CreateUsageEventRequestDataValue],
        timestamp: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        idempotency_key : str
            The idempotency key for the usage event. This ensures that the same event is not processed multiple times.

        event_name : str
            The name of the event. This is used by pricing metrics to aggregate usage events.

        subject_id : str
            The ID of the subject that the usage event is for.

        data : typing.Dict[str, CreateUsageEventRequestDataValue]
            The data of the usage event. This should contain any data that is needed to aggregate the usage event.

        timestamp : typing.Optional[dt.datetime]
            The timestamp of the usage event. It is highly recommended to provide a timestamp. If not provided, the current timestamp will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "usage-events",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "event_name": event_name,
                "subject_id": subject_id,
                "timestamp": timestamp,
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=typing.Dict[str, CreateUsageEventRequestDataValue], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawUsageEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_usage_event(
        self,
        *,
        idempotency_key: str,
        event_name: str,
        subject_id: str,
        data: typing.Dict[str, CreateUsageEventRequestDataValue],
        timestamp: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        idempotency_key : str
            The idempotency key for the usage event. This ensures that the same event is not processed multiple times.

        event_name : str
            The name of the event. This is used by pricing metrics to aggregate usage events.

        subject_id : str
            The ID of the subject that the usage event is for.

        data : typing.Dict[str, CreateUsageEventRequestDataValue]
            The data of the usage event. This should contain any data that is needed to aggregate the usage event.

        timestamp : typing.Optional[dt.datetime]
            The timestamp of the usage event. It is highly recommended to provide a timestamp. If not provided, the current timestamp will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "usage-events",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "event_name": event_name,
                "subject_id": subject_id,
                "timestamp": timestamp,
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=typing.Dict[str, CreateUsageEventRequestDataValue], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.create_subject_response import CreateSubjectResponse
from ..types.http_validation_error import HttpValidationError
from ..types.list_subjects_response import ListSubjectsResponse
from ..types.subject_resource import SubjectResource

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSubjectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_subjects(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListSubjectsResponse]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        offset : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListSubjectsResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "subjects",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListSubjectsResponse,
                    parse_obj_as(
                        type_=ListSubjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_subject(
        self,
        *,
        external_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateSubjectResponse]:
        """
        Parameters
        ----------
        external_id : typing.Optional[str]
            The ID of the subject in your system. If provided, you may use pass it to the API in place of the subject ID. Must be unique.

        name : typing.Optional[str]
            The name of the subject. Used for display in the dashboard.

        email : typing.Optional[str]
            The email of the subject. Must be a valid email address.

        metadata : typing.Optional[typing.Dict[str, str]]
            Additional metadata about the subject. You may use this to store any custom data about the subject.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateSubjectResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "subjects",
            method="POST",
            json={
                "external_id": external_id,
                "name": name,
                "email": email,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSubjectResponse,
                    parse_obj_as(
                        type_=CreateSubjectResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_subject(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SubjectResource]:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubjectResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"subjects/{jsonable_encoder(subject_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubjectResource,
                    parse_obj_as(
                        type_=SubjectResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_subject(
        self,
        subject_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SubjectResource]:
        """
        Parameters
        ----------
        subject_id : str

        name : typing.Optional[str]
            The name of the subject. Used for display in the dashboard.

        email : typing.Optional[str]
            The email of the subject. Must be a valid email address.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional metadata about the subject. You may use this to store any custom data about the subject.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubjectResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"subjects/{jsonable_encoder(subject_id)}",
            method="PUT",
            json={
                "name": name,
                "email": email,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubjectResource,
                    parse_obj_as(
                        type_=SubjectResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_subject(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"subjects/{jsonable_encoder(subject_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSubjectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_subjects(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListSubjectsResponse]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        offset : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListSubjectsResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "subjects",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListSubjectsResponse,
                    parse_obj_as(
                        type_=ListSubjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_subject(
        self,
        *,
        external_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateSubjectResponse]:
        """
        Parameters
        ----------
        external_id : typing.Optional[str]
            The ID of the subject in your system. If provided, you may use pass it to the API in place of the subject ID. Must be unique.

        name : typing.Optional[str]
            The name of the subject. Used for display in the dashboard.

        email : typing.Optional[str]
            The email of the subject. Must be a valid email address.

        metadata : typing.Optional[typing.Dict[str, str]]
            Additional metadata about the subject. You may use this to store any custom data about the subject.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateSubjectResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "subjects",
            method="POST",
            json={
                "external_id": external_id,
                "name": name,
                "email": email,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSubjectResponse,
                    parse_obj_as(
                        type_=CreateSubjectResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_subject(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SubjectResource]:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubjectResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"subjects/{jsonable_encoder(subject_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubjectResource,
                    parse_obj_as(
                        type_=SubjectResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_subject(
        self,
        subject_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SubjectResource]:
        """
        Parameters
        ----------
        subject_id : str

        name : typing.Optional[str]
            The name of the subject. Used for display in the dashboard.

        email : typing.Optional[str]
            The email of the subject. Must be a valid email address.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional metadata about the subject. You may use this to store any custom data about the subject.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubjectResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"subjects/{jsonable_encoder(subject_id)}",
            method="PUT",
            json={
                "name": name,
                "email": email,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubjectResource,
                    parse_obj_as(
                        type_=SubjectResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_subject(
        self, subject_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        subject_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"subjects/{jsonable_encoder(subject_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class PricingMetricAggregation_Count(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["count"] = "count"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PricingMetricAggregation_Custom(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["custom"] = "custom"
    custom_expression: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PricingMetricAggregation_Last(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["last"] = "last"
    value_field: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PricingMetricAggregation_Max(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["max"] = "max"
    value_field: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PricingMetricAggregation_Sum(UniversalBaseModel):
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    aggregation_type: typing.Literal["sum"] = "sum"
    value_field: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


PricingMetricAggregation = typing.Union[
    PricingMetricAggregation_Count,
    PricingMetricAggregation_Custom,
    PricingMetricAggregation_Last,
    PricingMetricAggregation_Max,
    PricingMetricAggregation_Sum,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.create_fixed_rate_request import CreateFixedRateRequest
from ..types.http_validation_error import HttpValidationError
from ..types.list_rate_cards_response import ListRateCardsResponse
from ..types.rate_card_resource import RateCardResource
from .types.create_rate_card_request_billing_interval import CreateRateCardRequestBillingInterval
from .types.create_rate_card_request_usage_based_rates_item import CreateRateCardRequestUsageBasedRatesItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRateCardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_rate_cards(
        self, *, limit: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListRateCardsResponse]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListRateCardsResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "rate-cards",
            method="GET",
            params={
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListRateCardsResponse,
                    parse_obj_as(
                        type_=ListRateCardsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_rate_card(
        self,
        *,
        name: str,
        billing_interval: CreateRateCardRequestBillingInterval,
        description: typing.Optional[str] = OMIT,
        fixed_rates: typing.Optional[typing.Sequence[CreateFixedRateRequest]] = OMIT,
        usage_based_rates: typing.Optional[typing.Sequence[CreateRateCardRequestUsageBasedRatesItem]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RateCardResource]:
        """
        Parameters
        ----------
        name : str
            The name of the rate card displayed to the customer.

        billing_interval : CreateRateCardRequestBillingInterval
            How often the customer will be billed for this rate card.

        description : typing.Optional[str]
            The description of the rate card displayed to the customer.

        fixed_rates : typing.Optional[typing.Sequence[CreateFixedRateRequest]]
            The fixed rates of the rate card. These are billed at the start of each billing cycle.

        usage_based_rates : typing.Optional[typing.Sequence[CreateRateCardRequestUsageBasedRatesItem]]
            The usage based rates of the rate card. These are billed at the end of each billing cycle.

        metadata : typing.Optional[typing.Dict[str, str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RateCardResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "rate-cards",
            method="POST",
            json={
                "name": name,
                "description": description,
                "billing_interval": billing_interval,
                "fixed_rates": convert_and_respect_annotation_metadata(
                    object_=fixed_rates, annotation=typing.Sequence[CreateFixedRateRequest], direction="write"
                ),
                "usage_based_rates": convert_and_respect_annotation_metadata(
                    object_=usage_based_rates,
                    annotation=typing.Sequence[CreateRateCardRequestUsageBasedRatesItem],
                    direction="write",
                ),
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RateCardResource,
                    parse_obj_as(
                        type_=RateCardResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_rate_card(
        self, rate_card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RateCardResource]:
        """
        Parameters
        ----------
        rate_card_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RateCardResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"rate-cards/{jsonable_encoder(rate_card_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RateCardResource,
                    parse_obj_as(
                        type_=RateCardResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRateCardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_rate_cards(
        self, *, limit: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListRateCardsResponse]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListRateCardsResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "rate-cards",
            method="GET",
            params={
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListRateCardsResponse,
                    parse_obj_as(
                        type_=ListRateCardsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_rate_card(
        self,
        *,
        name: str,
        billing_interval: CreateRateCardRequestBillingInterval,
        description: typing.Optional[str] = OMIT,
        fixed_rates: typing.Optional[typing.Sequence[CreateFixedRateRequest]] = OMIT,
        usage_based_rates: typing.Optional[typing.Sequence[CreateRateCardRequestUsageBasedRatesItem]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RateCardResource]:
        """
        Parameters
        ----------
        name : str
            The name of the rate card displayed to the customer.

        billing_interval : CreateRateCardRequestBillingInterval
            How often the customer will be billed for this rate card.

        description : typing.Optional[str]
            The description of the rate card displayed to the customer.

        fixed_rates : typing.Optional[typing.Sequence[CreateFixedRateRequest]]
            The fixed rates of the rate card. These are billed at the start of each billing cycle.

        usage_based_rates : typing.Optional[typing.Sequence[CreateRateCardRequestUsageBasedRatesItem]]
            The usage based rates of the rate card. These are billed at the end of each billing cycle.

        metadata : typing.Optional[typing.Dict[str, str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RateCardResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "rate-cards",
            method="POST",
            json={
                "name": name,
                "description": description,
                "billing_interval": billing_interval,
                "fixed_rates": convert_and_respect_annotation_metadata(
                    object_=fixed_rates, annotation=typing.Sequence[CreateFixedRateRequest], direction="write"
                ),
                "usage_based_rates": convert_and_respect_annotation_metadata(
                    object_=usage_based_rates,
                    annotation=typing.Sequence[CreateRateCardRequestUsageBasedRatesItem],
                    direction="write",
                ),
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RateCardResource,
                    parse_obj_as(
                        type_=RateCardResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_rate_card(
        self, rate_card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RateCardResource]:
        """
        Parameters
        ----------
        rate_card_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RateCardResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"rate-cards/{jsonable_encoder(rate_card_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RateCardResource,
                    parse_obj_as(
                        type_=RateCardResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .aggregation import Aggregation


class PricingMetricResource(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the pricing metric.
    """

    name: str = pydantic.Field()
    """
    The name of the pricing metric.
    """

    event_name: str = pydantic.Field()
    """
    The event name that the pricing metric is computed on.
    """

    aggregation: Aggregation = pydantic.Field()
    """
    The aggregation function used to compute the value of the pricing metric.
    """

    unit: str = pydantic.Field()
    """
    The unit of the value computed by the pricing metric.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .amount import Amount
from .package_price_output_rounding_behavior import PackagePriceOutputRoundingBehavior


class Price_Flat(UniversalBaseModel):
    price_type: typing.Literal["flat"] = "flat"
    amount: Amount

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Price_Package(UniversalBaseModel):
    price_type: typing.Literal["package"] = "package"
    amount: Amount
    package_units: int
    rounding_behavior: PackagePriceOutputRoundingBehavior

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


Price = typing.Union[Price_Flat, Price_Package]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.list_pricing_metrics_response import ListPricingMetricsResponse
from ..types.period import Period
from ..types.pricing_metric_resource import PricingMetricResource
from ..types.pricing_metric_summary_resource import PricingMetricSummaryResource
from .types.pricing_metric_aggregation import PricingMetricAggregation

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPricingMetricsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_pricing_metrics(
        self, *, limit: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListPricingMetricsResponse]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListPricingMetricsResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "pricing-metrics",
            method="GET",
            params={
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListPricingMetricsResponse,
                    parse_obj_as(
                        type_=ListPricingMetricsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_pricing_metric(
        self,
        *,
        name: str,
        event_name: str,
        aggregation: PricingMetricAggregation,
        unit: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PricingMetricResource]:
        """
        Parameters
        ----------
        name : str
            The name of the pricing metric.

        event_name : str
            The name of the event that the pricing metric is computed on.

        aggregation : PricingMetricAggregation
            The aggregation function used to compute the value of the pricing metric.

        unit : str
            Unit of measurement for the pricing metric.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PricingMetricResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "pricing-metrics",
            method="POST",
            json={
                "name": name,
                "event_name": event_name,
                "aggregation": convert_and_respect_annotation_metadata(
                    object_=aggregation, annotation=PricingMetricAggregation, direction="write"
                ),
                "unit": unit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PricingMetricResource,
                    parse_obj_as(
                        type_=PricingMetricResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_pricing_metric(
        self, pricing_metric_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PricingMetricResource]:
        """
        Parameters
        ----------
        pricing_metric_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PricingMetricResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"pricing-metrics/{jsonable_encoder(pricing_metric_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PricingMetricResource,
                    parse_obj_as(
                        type_=PricingMetricResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_pricing_metric_summary(
        self,
        pricing_metric_id: str,
        *,
        subject_id: str,
        period: Period,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PricingMetricSummaryResource]:
        """
        Parameters
        ----------
        pricing_metric_id : str

        subject_id : str
            The ID of the subject that the summary should be computed for.

        period : Period
            The period that the summary should be computed over.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PricingMetricSummaryResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"pricing-metrics/{jsonable_encoder(pricing_metric_id)}/summary",
            method="POST",
            json={
                "subject_id": subject_id,
                "period": convert_and_respect_annotation_metadata(object_=period, annotation=Period, direction="write"),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PricingMetricSummaryResource,
                    parse_obj_as(
                        type_=PricingMetricSummaryResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPricingMetricsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_pricing_metrics(
        self, *, limit: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListPricingMetricsResponse]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListPricingMetricsResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "pricing-metrics",
            method="GET",
            params={
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListPricingMetricsResponse,
                    parse_obj_as(
                        type_=ListPricingMetricsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_pricing_metric(
        self,
        *,
        name: str,
        event_name: str,
        aggregation: PricingMetricAggregation,
        unit: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PricingMetricResource]:
        """
        Parameters
        ----------
        name : str
            The name of the pricing metric.

        event_name : str
            The name of the event that the pricing metric is computed on.

        aggregation : PricingMetricAggregation
            The aggregation function used to compute the value of the pricing metric.

        unit : str
            Unit of measurement for the pricing metric.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PricingMetricResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "pricing-metrics",
            method="POST",
            json={
                "name": name,
                "event_name": event_name,
                "aggregation": convert_and_respect_annotation_metadata(
                    object_=aggregation, annotation=PricingMetricAggregation, direction="write"
                ),
                "unit": unit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PricingMetricResource,
                    parse_obj_as(
                        type_=PricingMetricResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_pricing_metric(
        self, pricing_metric_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PricingMetricResource]:
        """
        Parameters
        ----------
        pricing_metric_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PricingMetricResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"pricing-metrics/{jsonable_encoder(pricing_metric_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PricingMetricResource,
                    parse_obj_as(
                        type_=PricingMetricResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_pricing_metric_summary(
        self,
        pricing_metric_id: str,
        *,
        subject_id: str,
        period: Period,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PricingMetricSummaryResource]:
        """
        Parameters
        ----------
        pricing_metric_id : str

        subject_id : str
            The ID of the subject that the summary should be computed for.

        period : Period
            The period that the summary should be computed over.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PricingMetricSummaryResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"pricing-metrics/{jsonable_encoder(pricing_metric_id)}/summary",
            method="POST",
            json={
                "subject_id": subject_id,
                "period": convert_and_respect_annotation_metadata(object_=period, annotation=Period, direction="write"),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PricingMetricSummaryResource,
                    parse_obj_as(
                        type_=PricingMetricSummaryResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_subscription_checkout_session_response import CreateSubscriptionCheckoutSessionResponse
from .raw_client import AsyncRawCheckoutClient, RawCheckoutClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CheckoutClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCheckoutClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCheckoutClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCheckoutClient
        """
        return self._raw_client

    def create_subscription_checkout_session(
        self,
        *,
        subject_id: str,
        rate_card_id: str,
        success_callback_url: str,
        cancel_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSubscriptionCheckoutSessionResponse:
        """
        Parameters
        ----------
        subject_id : str
            The ID of the subject to create the checkout for.

        rate_card_id : str
            The ID of the rate card to subscribe to.

        success_callback_url : str
            The URL to redirect to after the checkout is successful.

        cancel_url : typing.Optional[str]
            The URL to redirect to after the checkout is cancelled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSubscriptionCheckoutSessionResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.checkout.create_subscription_checkout_session(
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            success_callback_url="https://example.com/callback",
        )
        """
        _response = self._raw_client.create_subscription_checkout_session(
            subject_id=subject_id,
            rate_card_id=rate_card_id,
            success_callback_url=success_callback_url,
            cancel_url=cancel_url,
            request_options=request_options,
        )
        return _response.data


class AsyncCheckoutClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCheckoutClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCheckoutClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCheckoutClient
        """
        return self._raw_client

    async def create_subscription_checkout_session(
        self,
        *,
        subject_id: str,
        rate_card_id: str,
        success_callback_url: str,
        cancel_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSubscriptionCheckoutSessionResponse:
        """
        Parameters
        ----------
        subject_id : str
            The ID of the subject to create the checkout for.

        rate_card_id : str
            The ID of the rate card to subscribe to.

        success_callback_url : str
            The URL to redirect to after the checkout is successful.

        cancel_url : typing.Optional[str]
            The URL to redirect to after the checkout is cancelled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSubscriptionCheckoutSessionResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.checkout.create_subscription_checkout_session(
                subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
                rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
                success_callback_url="https://example.com/callback",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_subscription_checkout_session(
            subject_id=subject_id,
            rate_card_id=rate_card_id,
            success_callback_url=success_callback_url,
            cancel_url=cancel_url,
            request_options=request_options,
        )
        return _response.data
